@@ -3,7 +3,9 @@ use std::collections::HashSet;
 use proc_macro2::{Span, TokenStream};
 use quote::quote;
 
-use crate::attr::{ContainerAttrs, EnumVariantAttrs, FieldAttrs, TypeDefault};
+use crate::attr::{
+    ensure_no_field_attrs, ContainerAttrs, EnumVariantAttrs, FieldAttrs, TypeDefault,
+};
 use crate::bound::{where_clause_with_bound, with_lifetime_bound};
 
 pub fn derive_deserialize(input: &mut syn::DeriveInput) -> syn::Result<TokenStream> {
@@ -12,6 +14,10 @@ pub fn derive_deserialize(input: &mut syn::DeriveInput) -> syn::Result<TokenStre
             fields: syn::Fields::Named(fields),
             ..
         }) => derive_struct(input, fields),
+        syn::Data::Struct(syn::DataStruct {
+            fields: syn::Fields::Unnamed(fields),
+            ..
+        }) if fields.unnamed.len() == 1 => derive_newtype_struct(input, &fields.unnamed[0]),
         syn::Data::Enum(enumeration) => derive_enum(input, enumeration),
         _ => panic!("only structs with named fields are supported"),
     }
@@ -430,3 +436,98 @@ pub fn derive_enum(
         };
     })
 }
+
+fn derive_newtype_struct(input: &syn::DeriveInput, field: &syn::Field) -> syn::Result<TokenStream> {
+    let ident = &input.ident;
+    let (impl_generics, ty_generics, where_clause) = input.generics.split_for_impl();
+    let dummy = syn::Ident::new(
+        &format!("_DESER_DESERIALIZE_IMPL_FOR_{}", ident),
+        Span::call_site(),
+    );
+
+    // TODO: we want to report the type name here but the current descriptor
+    // interface does not let us.  https://github.com/mitsuhiko/deser/issues/8
+    let container_attrs = ContainerAttrs::of(input)?;
+    let _type_name = container_attrs.container_name();
+
+    ensure_no_field_attrs(field)?;
+
+    let field_type = &field.ty;
+
+    let wrapper_generics = with_lifetime_bound(&input.generics, "'__a");
+    let (wrapper_impl_generics, wrapper_ty_generics, _) = wrapper_generics.split_for_impl();
+    let bound = syn::parse_quote!(::deser::Serialize);
+    let bounded_where_clause = where_clause_with_bound(&input.generics, bound);
+
+    Ok(quote! {
+        #[allow(non_upper_case_globals)]
+        const #dummy: () = {
+            struct __Sink #wrapper_impl_generics #where_clause {
+                slot: &'__a mut ::deser::__derive::Option<#ident #ty_generics>,
+                sink: ::deser::de::OwnedSink<#field_type #ty_generics>,
+            }
+
+            #[automatically_derived]
+            impl #impl_generics ::deser::de::Deserialize for #ident #ty_generics #bounded_where_clause {
+                fn deserialize_into(
+                    __slot: &mut ::deser::__derive::Option<Self>
+                ) -> ::deser::de::SinkHandle {
+                    ::deser::de::SinkHandle::boxed(__Sink {
+                        slot: __slot,
+                        sink: ::deser::de::OwnedSink::deserialize(),
+                    })
+                }
+            }
+
+            impl #wrapper_impl_generics ::deser::de::Sink for __Sink #wrapper_ty_generics #bounded_where_clause {
+                fn atom(&mut self, __atom: ::deser::Atom, __state: &::deser::de::DeserializerState)
+                    -> ::deser::__derive::Result<()>
+                {
+                    self.sink.borrow_mut().atom(__atom, __state)
+                }
+
+                fn map(&mut self, __state: &::deser::de::DeserializerState) -> ::deser::__derive::Result<()> {
+                    self.sink.borrow_mut().map(__state)
+                }
+
+                fn seq(&mut self, __state: &::deser::de::DeserializerState) -> ::deser::__derive::Result<()>  {
+                    self.sink.borrow_mut().seq(__state)
+                }
+
+                fn next_key(&mut self, __state: &::deser::de::DeserializerState)
+                    -> ::deser::__derive::Result<::deser::de::SinkHandle>
+                {
+                    self.sink.borrow_mut().next_key(__state)
+                }
+
+                fn next_value(&mut self, __state: &::deser::de::DeserializerState)
+                    -> ::deser::__derive::Result<::deser::de::SinkHandle>
+                {
+                    self.sink.borrow_mut().next_value(__state)
+                }
+
+                fn value_for_key(
+                    &mut self,
+                    __key: &str,
+                    __state: &::deser::de::DeserializerState,
+                ) -> ::deser::__derive::Result<::deser::__derive::Option<::deser::de::SinkHandle>> {
+                    self.sink.borrow_mut().value_for_key(__key, __state)
+                }
+
+                fn finish(&mut self, __state: &::deser::de::DeserializerState) -> ::deser::__derive::Result<()> {
+                    self.sink.borrow_mut().finish(__state)?;
+                    *self.slot = self.sink.take().map(#ident);
+                    Ok(())
+                }
+
+                fn descriptor(&self) -> &dyn ::deser::Descriptor {
+                    self.sink.borrow().descriptor()
+                }
+
+                fn expecting(&self) -> ::deser::__derive::StrCow<'_> {
+                    self.sink.borrow().expecting()
+                }
+            }
+        };
+    })
+}
